<h3 style="font-size: 30px;"><a href="{{ pathto(master_doc) }}">crispy-forms</a></h3>
<p>
  <iframe src="https://ghbtns.com/github-btn.html?user=django-crispy-forms&repo=django-crispy-forms&type=star&count=true&size=large" frameborder="0" scrolling="0" width="160px" height="30px"></iframe>
</p>

<p>
  django-crispy-forms is a Django application that lets you easily build, customize and reuse forms using your favorite CSS framework, without writing template code and without having to take care of annoying details. You are currently looking at the documentation of the development release.
</p>
